@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package Dist::Zilla::PluginBundle::Git::VersionManager; # git description: v0.001-6-gffba85c
+package Dist::Zilla::PluginBundle::Git::VersionManager; # git description: v0.002-6-g3439e40
 # vim: set ts=8 sts=4 sw=4 tw=115 et :
 # ABSTRACT: A plugin bundle that manages your version in git
 # KEYWORDS: bundle distribution git version Changes increment
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 use Moose;
 with
@@ -19,6 +19,14 @@ use Moose::Util::TypeConstraints qw(subtype where class_type);
 use CPAN::Meta::Requirements;
 use namespace::autoclean;
 
+has bump_only_matching_versions => (
+    is => 'ro',
+    isa => 'Bool',
+    init_arg => undef,
+    lazy => 1,
+    default => sub { $_[0]->payload->{bump_only_matching_versions} },
+);
+
 has changes_version_columns => (
     is => 'ro', isa => subtype('Int', where { $_ > 0 && $_ < 20 }),
     init_arg => undef,
@@ -56,6 +64,9 @@ sub configure
 {
     my $self = shift;
 
+    die 'you cannot change the distribution version with $V along with bump_only_matching_versions: update the .pm file(s) first'
+        if $ENV{V} and $self->bump_only_matching_versions;
+
     my $fallback_version_provider =
         $self->payload->{'RewriteVersion::Transitional.fallback_version_provider'}
             // 'Git::NextVersion';  # TODO: move this default to an attribute; be careful of overlays
@@ -73,7 +84,9 @@ sub configure
 
     $self->add_plugins(
         # VersionProvider (and a file munger, for the transitional usecase)
-        [ 'RewriteVersion::Transitional' => { ':version' => '0.004' } ],
+        $self->bump_only_matching_versions
+            ? [ 'VersionFromMainModule' ]
+            : [ 'RewriteVersion::Transitional' => { ':version' => '0.004' } ],
 
         [ 'MetaProvides::Update' ],
 
@@ -84,7 +97,14 @@ sub configure
                 allow_dirty => [ $self->commit_files_after_release ],
             } ],
         [ 'Git::Tag' ],
-        [ 'BumpVersionAfterRelease::Transitional' => { ':version' => '0.004' } ],
+
+        # for all_matching => 1, we presume the author already has versions set up the way he wants them (and for
+        # consistency with the removal of RewriteVersion above), so we do not bother with it;
+        # this also lets us specify the minimum necessary version for the feature.
+        $self->bump_only_matching_versions
+            ? [ 'BumpVersionAfterRelease' => { ':version' => '0.016', all_matching => 1 } ]
+            : [ 'BumpVersionAfterRelease::Transitional' => { ':version' => '0.004' } ],
+
         [ 'NextRelease'         => {
                 ':version' => '5.033',
                 time_zone => 'UTC',
@@ -92,7 +112,15 @@ sub configure
             } ],
         [ 'Git::Commit'         => 'post-release commit' => {
                 ':version' => '2.020',
-                allow_dirty => [ 'Changes' ],
+                allow_dirty => [
+                    'Changes',
+                    !exists($self->payload->{'BumpVersionAfterRelease::Transitional.munge_makefile_pl'})
+                            || $self->payload->{'BumpVersionAfterRelease::Transitional.munge_makefile_pl'}
+                        ? 'Makefile.PL' : (),
+                    !exists($self->payload->{'BumpVersionAfterRelease::Transitional.munge_build_pl'})
+                            || $self->payload->{'BumpVersionAfterRelease::Transitional.munge_build_pl'}
+                        ? 'Build.PL' : (),
+                ],
                 allow_dirty_match => [ '^lib/.*\.pm$' ],
                 commit_msg => 'increment $VERSION after %v release'
             } ],
@@ -146,7 +174,7 @@ Dist::Zilla::PluginBundle::Git::VersionManager - A plugin bundle that manages yo
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -172,7 +200,10 @@ release commit to C<git>, which is then tagged, and then the C<$VERSION> in code
 change is committed.  The default options used for the plugins are carefully curated to work together, but all
 can be customized or overridden (see below).
 
-It is equivalent to the following configuration directly in a F<dist.ini>:
+Modules without C<$VERSION> declarations will have them added for the release, with that change also committed
+back to the local repository.
+
+When no custom options are passed, is equivalent to the following configuration directly in a F<dist.ini>:
 
     [RewriteVersion::Transitional]
     :version = 0.004
@@ -214,6 +245,29 @@ It is equivalent to the following configuration directly in a F<dist.ini>:
 
 =for stopwords CopyFilesFromRelease
 
+=head2 bump_only_matching_versions
+
+If your distribution has many module files, and not all of them have a C<$VERSION> declaration that matches the
+main module (and distribution) version, set this option to true. This has two effects that differ from the default
+case:
+
+=over 4
+
+=item *
+
+while preparing the build and release, I<no> module C<$VERSION> declarations will be altered to match the distribution version (therefore they must be set to the desired values in advance). Consequently, attempting to alter the distribution version with C<V=...> will result in a fatal error.
+
+=item *
+
+after the release, only module C<$VERSION> declarations that match the release version will see their values incremented. All C<$VERSION>s that do not match will be left alone: you must manage them manually. Likewise, no missing $VERSIONs will be added.
+
+=back
+
+Defaults to false (meaning all modules will have their C<$VERSION> declarations synchronized before release and
+incremented after release).
+
+First available in version 0.003.
+
 =head2 commit_files_after_release
 
 File(s), which are expected to exist in the repository directory,
